#include "gbox.h"
#include "gbox_mailbox.h"

#include <stdlib.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/file.h>
#include <fcntl.h>
#include <sys/types.h>

MailBoxList *gbox_mailbox_list_new() {
  MailBoxList *list;
  
  list = (MailBoxList *) g_malloc(sizeof(MailBoxList));
  
  return list;
}

MailBoxList *gbox_mailbox_list_copy(MailBoxList *list) {
  MailBoxList *new_list = gbox_mailbox_list_new();
  MailBox     *walk = list->first_mb;
  
  if (!list) {
    gbox_mailbox_list_free(new_list);
    return NULL;
  }
    
  while (walk) {
    gbox_mailbox_add(new_list, walk);
    walk = walk->next;
  }
  
  return new_list;
}


void gbox_mailbox_list_free(MailBoxList *list) {
  while (list->first_mb)
    gbox_mailbox_remove(list, list->first_mb->mailfile);

  g_free(list);
}

MailBox *gbox_mailbox_new(char *file, char *nick, int interval, int prio, int show, int menu_type) {
    MailBox *new_mb = g_malloc(sizeof(MailBox));

    new_mb->next = NULL;
    new_mb->mailfile = g_strdup(file);
    new_mb->nickname = g_strdup(nick);
    new_mb->size = 0;
    new_mb->interval = interval;
    new_mb->priority = prio;
    new_mb->show_in_menu = show;
    new_mb->menu_type = menu_type;
    new_mb->mail_count = 0;

    return new_mb;
}

void gbox_mailbox_add(MailBoxList *list, MailBox *new_box) {
    if (list->last_mb) list->last_mb->next = new_box;
    list->last_mb = new_box;
    if (!list->first_mb) list->first_mb = new_box;
}

void gbox_mailentries_free(MailEntry **mail){
  MailEntry *current = NULL;
  
  if ((*mail) == NULL) return;

  while ((*mail) != NULL) {
    current = (*mail);
    g_free ((*mail)->from);
    g_free ((*mail)->subject);
    (*mail) = (*mail)->next;
    g_free (current);    
  }
}

int gbox_mailbox_count_mails(char *file, MailEntry **mail, int with_data) {
  struct flock ld;
  int fd;
  FILE *f = 0;
  char buf[1024];
  char *boundary = NULL;
  char *boundary_end = NULL;
  char *pos1 = NULL, *pos2 = NULL;
  int count = 0;
  int ignore = 0;
  int new_mail = 2;
  char *lf;

  MailEntry *newMail = NULL;
  MailEntry *lastMail = NULL;

 
  ld.l_type = F_RDLCK;
  ld.l_whence = SEEK_SET;
  ld.l_start = 0;
  ld.l_len = 0;
  
  if (with_data) 
    (*mail) = NULL;
  
  if ((fd = open(file,O_RDONLY)) != -1) {
    while (fcntl (fd, F_SETLKW, &ld) == -1) ;
    
    if ((f = fdopen(fd,"r"))) {
      while (fgets(buf,sizeof(buf),f) != 0) {
	
	if (with_data) {
	  
	  if (new_mail == 2) {
	    newMail = (MailEntry*)malloc(sizeof(MailEntry));
	    newMail->from = newMail->subject = NULL;
	    newMail->next = NULL;
	    
	    if ((*mail) == NULL) {
	      (*mail) = newMail;
	      lastMail = newMail;
	    } else {
	      lastMail->next = newMail;
	      lastMail = newMail;
	    }
	    
	    new_mail = 0;
	  }
	  
	  if (!ignore && strncmp(buf,"From: ",6) == 0) {
	    /* extract From */
	    lf = strchr(buf,'\n');
	    if (lf != NULL) *lf = '\0';
	    lastMail->from=(char*)malloc((size_t)(strlen(buf)-5));
	    strcpy(lastMail->from,buf+6);
	    new_mail++;
	  }
	  
	  if (!ignore && strncmp(buf,"Subject: ",9) == 0) {
	    /* extract Subject */
	    lf = strchr(buf,'\n');
	    if (lf != NULL) *lf = '\0';
	    lastMail->subject=(char*)malloc((size_t)(strlen(buf)-8));
	    strcpy(lastMail->subject,buf+9);
	    new_mail++;
	  }
	  
	  if (new_mail == 1 && buf[0] == 0x0A) {
	    new_mail = 2;
	  }
	} /* of with_data */
	
	if (!ignore && strncmp(buf,"From ",5) == 0) {
	  count++;
	}

	
	if (!ignore && strncmp(buf,"Content-Type: ",14) == 0){ 
/*  	  printf("Content: %s\n", buf); */
	  
	  if ((pos1 = strstr(buf,"boundary=")) != 0) {
	    /* boundary in the same line */
	    pos1 += 9;
	    ignore = 1;
/*  	    pos1 = strchr(buf,'"'); */
/*  	    pos2 = strrchr(buf,'"'); */
	    
	    pos2 = pos1 + 1;
	    if (pos1[0] == '"')
	      while (*pos2) if (*pos2 != '"') pos2++; else break;
	    else
	      while (*pos2) if (*pos2 != '\n' && *pos2 != ' ') pos2++; else break;

	    if (pos1 != 0 && pos2 > pos1+1) {
	      boundary = (char*)malloc((size_t)(pos2-pos1 + 1));
	      strncpy(boundary,pos1+1,pos2-pos1-1);
	      boundary[pos2-pos1-1] = '\0';
	      boundary_end = (char*)malloc((size_t)(pos2-pos1+4));
	      sprintf(boundary_end, "--%s--", boundary);
/*  	      printf("(%s), (%s)\n", boundary, boundary_end); */
	    }
	    
	  } else {
	    /* boundary in the next line */ 
	    if (fgets(buf,sizeof(buf),f) != 0) {
/*  	      printf("Content: %s\n", buf); */
	      
	      if ((pos1 = strstr(buf,"boundary=")) != 0) {  
		pos1 += 9;
		ignore = 1;
		pos2 = pos1 + 1;
		if (pos1[0] == '"')
		  while (*pos2) if (*pos2 != '"') pos2++; else break;
		else
		  while (*pos2) if (*pos2 != '\n' && *pos2 != ' ') pos2++; else break;
		  
		if (pos1 != 0 && pos2 > pos1+1) {
		  boundary = (char*)malloc((size_t)(pos2-pos1 + 1));
		  strncpy(boundary,pos1+1,pos2-pos1-1);
		  boundary[pos2-pos1-1] = '\0';
		  boundary_end = (char*)malloc((size_t)(pos2-pos1+5));
		  sprintf(boundary_end, "--%s--", boundary);
/*  		  printf("(%s), (%s)\n", boundary, boundary_end); */
		}
	      }     
	    }
	  }
	}
	
	
	if ((ignore == 1) && (strncmp(buf,boundary_end,strlen(boundary_end)) == 0)) {
	  ignore = 0;
	  free(boundary); boundary = NULL;
	  free(boundary_end); boundary_end = NULL;
	}

	memset(buf, 0, sizeof(buf));
      }
      fclose (f);
    } else count = -1;
    ld.l_type = F_UNLCK;
    while (fcntl (fd, F_SETLKW, &ld) != -1) ;    
    close(fd);
  }else count = -1;
  if (boundary) {
	  free(boundary); boundary = NULL;
	  free(boundary_end); boundary_end = NULL;
  }
  
  return count;
}

int gbox_mailbox_list_count_with_new_mail(MailBoxList *list) {
  MailBox *walk = list->first_mb;
  int count = 0;
  
  while (walk) {
    if (walk->mail_count) count++;
    walk = walk->next;
  }
  
  return count;
}
 
int gbox_mailbox_list_count(MailBoxList *list) {
  MailBox *walk = list->first_mb;

  int count = 0;
  
  while (walk) {
    count++;
    walk = walk->next;
  }
  
  return count;
}

void gbox_mailbox_free(MailBox *box) {
    g_free(box->mailfile);
    g_free(box->nickname);

    g_free(box);
}

void gbox_mailbox_change(MailBoxList *list, char *file, MailBox* new_box) {
  MailBox *walk = list->first_mb;
  
  while (walk) {
    if (!strcmp(walk->mailfile, file)) break;
    walk = walk->next;
  }
  
  if (walk) {
    g_free(walk->mailfile);
    g_free(walk->nickname);
    walk->mailfile = g_strdup(new_box->mailfile);
    walk->nickname = g_strdup(new_box->nickname);
    walk->interval = new_box->interval;
    walk->priority = new_box->priority;
    walk->show_in_menu = new_box->show_in_menu;
    walk->menu_type = new_box->menu_type;
  }
}

void gbox_mailbox_remove(MailBoxList *list, char *file) {
  MailBox *walk = list->first_mb, *prev = NULL;
  
  while (walk) {
    if (!strcmp(walk->mailfile, file)) break;
    
    prev = walk;
    walk = walk->next;
  }
  
  if (!walk) return;
  
  if (!prev) {   // remove first;
    list->first_mb = list->first_mb->next;
  } else {
    prev->next = walk->next;
  }
  
  if (list->last_mb == walk) list->last_mb = prev;
  
  gbox_mailbox_free(walk);
}

void gbox_mailbox_remove_by_index(MailBoxList *list, int index) {
  gbox_mailbox_remove(list, gbox_mailbox_get(list, index)->mailfile);
}

MailBox *gbox_mailbox_get(MailBoxList *list, int index) {
  MailBox *walk = list->first_mb;
  int i = 0;
  
  while (walk) {
    if (i == index) break;
    i++;
    walk = walk->next;
  }

  return walk;
}
